@@ -95,10 +95,7 @@ class Syncer:
         operations = []
         try:
             conditions = self._create_key_value_pairs(
-                from_entity,
-                mapping.keys,
-                from_ds=from_ds,
-                to_ds=data_source,
+                from_entity, mapping.keys, from_ds=from_ds, to_ds=data_source,
             )
             found_entities = data_source.find(entity_type, conditions)
         except NotLinked as e:
@@ -113,10 +110,7 @@ class Syncer:
 
             try:
                 non_key_attributes = self._create_key_value_pairs(
-                    from_entity,
-                    mapping.attributes,
-                    from_ds=from_ds,
-                    to_ds=data_source,
+                    from_entity, mapping.attributes, from_ds=from_ds, to_ds=data_source,
                 )
                 to_entity = Syncer._create_entity(
                     conditions, non_key_attributes, entity_type, from_entity_timestamp
@@ -135,6 +129,13 @@ class Syncer:
                 )
                 return operations
 
+            if mapping.dirty_mode == DirtyMode.UPDATE:
+                setattr(from_entity, mapping.dirty_attribute_name, False)
+                logging.info(
+                    f"Removed {mapping.dirty_attribute_name} flag from {from_entity}"
+                )
+                from_ds.persist(from_entity)
+
             operations.append(
                 Addition(from_entity.__class__, entity_type, from_entity, to_entity)
             )
@@ -219,6 +220,16 @@ class Syncer:
             logging.debug(f"Setting timestamp of {entity} to {timestamp}")
             entity.update_timestamp(timestamp)
 
+    @staticmethod
+    def _is_dirty(mapping, entity):
+        try:
+            return getattr(entity, mapping.dirty_attribute_name)
+        except AttributeError as e:
+            logging.warning(
+                f"asked to read {mapping.dirty_attribute_name} from {entity} but no such attribute exists"
+            )
+            raise e
+
     def _update_linked_entity(
         self,
         from_ds: DataSource,
@@ -230,31 +241,13 @@ class Syncer:
     ):
         operations = []
 
-        def is_dirty(entity):
-            try:
-                return getattr(entity, mapping.dirty_attribute_name)
-            except AttributeError as e:
-                logging.warn(
-                    f"asked to read {mapping.dirty_attribute_name} from {entity} but no such attribute exists"
-                )
-                raise e
-
-        if mapping.dirty_mode == DirtyMode.UPDATE and not is_dirty(from_entity):
-            return operations
-
         try:
             attributes = {
                 **self._create_key_value_pairs(
-                    from_entity,
-                    mapping.keys,
-                    from_ds=from_ds,
-                    to_ds=to_ds,
+                    from_entity, mapping.keys, from_ds=from_ds, to_ds=to_ds,
                 ),
                 **self._create_key_value_pairs(
-                    from_entity,
-                    mapping.attributes,
-                    from_ds=from_ds,
-                    to_ds=to_ds,
+                    from_entity, mapping.attributes, from_ds=from_ds, to_ds=to_ds,
                 ),
             }
         except NotLinked as e:
@@ -292,7 +285,9 @@ class Syncer:
             else:
                 to_entity = refreshed_to_entity
 
-            if mapping.dirty_mode == DirtyMode.DO_NOT_OVERWRITE and is_dirty(to_entity):
+            if mapping.dirty_mode == DirtyMode.DO_NOT_OVERWRITE and self._is_dirty(
+                mapping, to_entity
+            ):
                 continue
 
             # guard that to_entity has last been changed before or at the same time as from_entity
@@ -332,15 +327,15 @@ class Syncer:
                 to_ds.persist(to_entity)
                 if mapping.dirty_mode == DirtyMode.UPDATE:
                     setattr(from_entity, mapping.dirty_attribute_name, False)
+                    logging.info(
+                        f"Removed {mapping.dirty_attribute_name} flag from {from_entity}"
+                    )
                     from_ds.persist(from_entity)
 
         return operations
 
     def _sync_from_to(
-        self,
-        from_ds: DataSource,
-        to_ds: DataSource,
-        mapping: Mapping,
+        self, from_ds: DataSource, to_ds: DataSource, mapping: Mapping,
     ):
         operations = []
         from_class, to_class = mapping.entity_types
@@ -384,6 +379,12 @@ class Syncer:
 
     def _sync_entity(self, from_ds, from_entity, to_ds, to_class, mapping):
         operations = []
+
+        if mapping.dirty_mode == DirtyMode.UPDATE and not self._is_dirty(
+            mapping, from_entity
+        ):
+            return operations
+
         others = self.link_provider.others(from_entity, from_ds, to_class, to_ds)
         if not others or not len(others):
             operations.extend(
@@ -394,12 +395,7 @@ class Syncer:
         else:
             operations.extend(
                 self._update_linked_entity(
-                    from_ds,
-                    from_entity,
-                    others,
-                    mapping,
-                    to_class,
-                    to_ds,
+                    from_ds, from_entity, others, mapping, to_class, to_ds,
                 )
             )
         return operations
